@@ -3,11 +3,12 @@ grammar JSON::Tiny::Grammar;
 
 rule TOP        { ^[ <object> | <array> ]$ }
 rule object     { '{' ~ '}' <pairlist>     }
-rule pairlist   { [ <pair> ** [ \, ]  ]?   }
+rule pairlist   { <pair>* % \,             }
 rule pair       { <string> ':' <value>     }
-rule array      { '[' ~ ']' [ <value> ** [ \, ] ]?  }
+rule array      { '[' ~ ']' <arraylist>    }
+rule arraylist  { <value>* % [ \, ]        }
 
-proto token value { <...> };
+proto token value {*};
 token value:sym<number> {
     '-'?
     [ 0 | <[1..9]> <[0..9]>* ]
@@ -26,14 +27,7 @@ token string {
 }
 
 token str {
-    [
-        <!before \t>
-        <!before \n>
-        <!before \\>
-        <!before \">
-        .
-    ]+
-#    <-["\\\t\n]>+
+    <-["\\\t\n]>+
 }
 
 token str_escape {
