@@ -5,7 +5,11 @@ use File::Find;
 use Shell::Command;
 
 class Panda::Fetcher does Pies::Fetcher {
-    has $!resources;
+    sub die (Pies::Project $p, $d) is hidden_from_backtrace {
+        X::Panda.new($p.name, 'fetch', $d).throw
+    }
+
+    has $.resources;
     method fetch (Pies::Project $p) {
         my $dest = $!resources.workdir($p);
         # the repo-* variants are kept for backwards compatibility only
@@ -17,8 +21,7 @@ class Panda::Fetcher does Pies::Fetcher {
                     $type = 'git';
                 }
                 default {
-                    die "Failed fetching {$p.name}, unable to determine "
-                      ~ "source-type with the source-url";
+                    die $p, "Unable to determine source-type using source-url";
                 }
             }
         }
@@ -26,12 +29,12 @@ class Panda::Fetcher does Pies::Fetcher {
             when 'git' {
                 if $dest.IO ~~ :d {
                     indir $dest, {
-                        run 'git pull -q'
-                        and die "Failed updating the {$p.name} repo";
+                        shell 'git pull -q'
+                        and die $p, "Failed updating the repo";
                     };
                 } else {
-                    run "git clone -q $url $dest"
-                        and die "Failed cloning the {$p.name} repo";
+                    shell "git clone -q $url $dest"
+                        and die $p, "Failed cloning the repo";
                 }
             }
             when 'local' {
@@ -45,12 +48,9 @@ class Panda::Fetcher does Pies::Fetcher {
                 }
             }
             default {
-                die "Failed fetching {$p.name}, "
-                    ~ "source-type $_ not supported";
+                die $p, "source-type $_ not supported";
             }
         }
-        # returns the directory where the module lies
-        # return {$Settings::srcdir} ~ "/$name";
     }
 }
 
