@@ -4,19 +4,21 @@ use File::Find;
 use Shell::Command;
 
 class Panda::Installer does Pies::Installer {
-    has $!resources;
-    has $!destdir;
+    sub die (Pies::Project $p, $d) is hidden_from_backtrace {
+        X::Panda.new($p.name, 'install', $d).throw
+    }
+
+    has $.resources;
+    has $.destdir;
 
     method install(Pies::Project $p) {
         indir $!resources.workdir($p), {
-            if 'Makefile'.IO ~~ :f {
-                run 'make install'
-                    and die "'make install' failed for {$p.name}";
-                return;
-            }
             if 'blib'.IO ~~ :d {
-                for find(dir => 'blib', type => 'file').list -> $i {
+                for find(dir => 'blib', name => /\.pm$/).list,
+                    find(dir => 'blib', name => /\.pir$/).list
+                    -> $i {
                     # .substr(5) to skip 'blib/'
+                    say "Installing $i";
                     mkpath "$!destdir/{$i.dir.substr(5)}";
                     $i.IO.copy("$!destdir/{$i.Str.substr(5)}");
                 }
